% _________________________________________________________________________ 
%function y = pam_sqrrc(Nss, LS, roff, S, tim_err, freq_err)
%  
% BPSK SIGNAL GENERATION (SQUARE-ROOT RAISED COSINE)
% BASE-BAND EQUIVALENT
%
% - Input Parameters -
%
%   Nss      : number of samples per symbol (typically, 4).
%   LS       : length of shaping pulse in symbols (typically, 10).
%   roff     : roll-off (typically 0.35 for satellite).
%   S        : BPSK symbols ("1" generates just one pulse).
%   tim_err  : normalized timing error (0=no error, 1=one symbol delay).
%   freq_err : discrete-time normalized frequency error (0=no error).
%
% - Output Parameters -
%
%   y        : output samples vector.
%
% _________________________________________________________________________
% function sqrrc_generator()
% pam_sqrrc(Nss, LS, roff, S, tim_err, frq)

apagar_boc;
encender_sqrrc;

nombre_senal = findobj(gcbf, 'Tag', 'panel_senal'); 
set(nombre_senal, 'Title', 'SQRRC');

res        = findobj(gcbf, 'Tag', 'res_senal');           
dur_senal  = findobj(gcbf, 'Tag', 'duracion_senal');  

h_roll_off = findobj(gcbf, 'Tag', 'roll_off');
h_nss      = findobj(gcbf, 'Tag', 'nss');
h_ls       = findobj(gcbf, 'Tag', 'ls');

if default == 1
        set(res, 'String', '20');
        set(dur_senal, 'String', '25');        
        set(h_roll_off, 'String', '0.35');        
        set(h_nss, 'String', '4');
        set(h_ls, 'String', '6');
end

% Tomamos los parámetros para este tipo de señal. O son los default o son
% los introducidos por el usuario

    roff= str2num(get(h_roll_off, 'String'));    
    Nss = str2num(get(h_nss, 'String'));
    LS  = str2num(get(h_ls, 'String'));

    S = 1;
    tim_err = 0;
    frq = 0;
% _________________________________________________________________________
% Initialize parameters

NS = length(S);                     	% number of symbols.
Lp = LS*Nss;     				    	% length of shaping pulse in samples

if( ((Lp/2) - ceil(Lp/2)) == 0 )        % ODD PATCHING
    Lp = Lp+1;
end;

Ly = (NS-1)*Nss+Lp;

if( nargin == 4 )
    tim_err = eps;
end

% _________________________________________________________________________
% Create pulse shaping pattern
% vector of time samples
t   =   -tim_err + 1e-6 + [-(Lp-1)/2:1:(Lp-1)/2]/Nss;	

% calculates square root raised cosine
p   =   (sin(pi*(1-roff)*t) + 4*roff*t.*cos(pi*(1+roff)*t))./(pi*t.*(1-(4*roff*t).^2));

% _________________________________________________________________________
% Generate output sequence

y = zeros(1,Ly);

for k=0:NS-1,			
    k1 = k*Nss+1;                   		% lower index
    k2 = k*Nss+Lp;                 			% higher index
    y(k1:k2) = y(k1:k2)+S(k+1)*p;  			% add new symbol        
end

% Include frequency error degradation.
frqerr = exp(i*(2*pi*frq*(0:(length(y)-1))));
y = y.*frqerr;

amplis = findobj(gcbf, 'Tag', 'amplitudes');
set(amplis, 'String',num2str(y));

num_muestras = findobj(gcbf, 'Tag', 'num_muestras');
set(num_muestras, 'String', length(y),'BackgroundColor', [0.5 0.5 0.5], 'Enable', 'inactive');

h_gnss = findobj(gcbf, 'Tag', 'desplegable_gnss');
z = get(h_gnss, 'userData');
j = z(1); 
set(h_gnss, 'UserData', [j,2]);  

% Dibujamos la señal
   
h1 = findall(gcf, 'Tag', 'panel_temporal_senal');
h_temp = subplot('Position', [0.10 0.20 0.85 0.72], 'Parent', h1);

stem(h_temp, y)

% Le damos unpoco de margen para que la gráfica *respire*
ylim(h_temp,[min([0 y*1.2]) max(y)*1.2])
xlim(h_temp, [0 length(y)+1])
grid(h_temp, 'on')
       
